searchNodes=[{"doc":"Struct and functions to represent LoR Cards.","ref":"Riot.LoR.Card.html","title":"Riot.LoR.Card","type":"module"},{"doc":"Represent a Card as a tuple of its set, faction id, and number. Examples iex&gt; card = % Riot.LoR.Card { set : 1 , fac : 0 , num : 1 } % Riot.LoR.Card { fac : 0 , num : 1 , set : 1 } iex&gt; Riot.LoR.Card . as_tuple ( card ) { 1 , 0 , 1 }","ref":"Riot.LoR.Card.html#as_tuple/1","title":"Riot.LoR.Card.as_tuple/1","type":"function"},{"doc":"Create a Card from a tuple of it's set, faction id, and number. Examples iex&gt; Riot.LoR.Card . new ( { 1 , 0 , 1 } ) % Riot.LoR.Card { fac : 0 , num : 1 , set : 1 }","ref":"Riot.LoR.Card.html#new/1","title":"Riot.LoR.Card.new/1","type":"function"},{"doc":"Create a Card from a card code string. Examples iex&gt; Riot.LoR.Card . parse! ( &quot;01DE001&quot; ) % Riot.LoR.Card { fac : 0 , num : 1 , set : 1 }","ref":"Riot.LoR.Card.html#parse!/1","title":"Riot.LoR.Card.parse!/1","type":"function"},{"doc":"Represent a Card as its card code string. Examples iex&gt; card = % Riot.LoR.Card { set : 1 , fac : 0 , num : 1 } % Riot.LoR.Card { fac : 0 , num : 1 , set : 1 } iex&gt; Riot.LoR.Card . to_code! ( card ) &quot;01DE001&quot;","ref":"Riot.LoR.Card.html#to_code!/1","title":"Riot.LoR.Card.to_code!/1","type":"function"},{"doc":"","ref":"Riot.LoR.Card.html#t:t/0","title":"Riot.LoR.Card.t/0","type":"type"},{"doc":"Type and functions to represent a LoR deck, i.e., cards and their respective counts.","ref":"Riot.LoR.Deck.html","title":"Riot.LoR.Deck","type":"module"},{"doc":"Add a Riot.LoR.Card count to a Riot.LoR.Deck . By default it will increment the count by 1. You can increment by any positive integer. Examples iex&gt; deck = Riot.LoR.Deck . new ( ) iex&gt; deck = Riot.LoR.Deck . add_card ( deck , % Riot.LoR.Card { set : 1 , fac : 0 , num : 1 } ) %{ % Riot.LoR.Card { fac : 0 , num : 1 , set : 1 } =&gt; 1 } iex&gt; deck = Riot.LoR.Deck . add_card ( deck , % Riot.LoR.Card { set : 1 , fac : 0 , num : 2 } , 2 ) %{ % Riot.LoR.Card { fac : 0 , num : 1 , set : 1 } =&gt; 1 , % Riot.LoR.Card { fac : 0 , num : 2 , set : 1 } =&gt; 2 } iex&gt; Riot.LoR.Deck . add_card ( deck , % Riot.LoR.Card { set : 1 , fac : 0 , num : 1 } , 2 ) %{ % Riot.LoR.Card { fac : 0 , num : 1 , set : 1 } =&gt; 3 , % Riot.LoR.Card { fac : 0 , num : 2 , set : 1 } =&gt; 2 }","ref":"Riot.LoR.Deck.html#add_card/3","title":"Riot.LoR.Deck.add_card/3","type":"function"},{"doc":"Add a raw card code to a Riot.LoR.Deck . By default it will increment the count by 1. You can intrement by any positive integer. This is a convenience wrapper around Riot.LoR.Deck.add_card/3 . Examples iex&gt; deck = Riot.LoR.Deck . new ( ) iex&gt; deck = Riot.LoR.Deck . add_card_code! ( deck , &quot;01DE001&quot; ) %{ % Riot.LoR.Card { fac : 0 , num : 1 , set : 1 } =&gt; 1 } iex&gt; Riot.LoR.Deck . add_card_code! ( deck , &quot;01DE001&quot; , 2 ) %{ % Riot.LoR.Card { fac : 0 , num : 1 , set : 1 } =&gt; 3 }","ref":"Riot.LoR.Deck.html#add_card_code!/3","title":"Riot.LoR.Deck.add_card_code!/3","type":"function"},{"doc":"Return a list of card codes and their counts. Examples iex&gt; deck = Riot.LoR.Deck . new ( ) %{ } iex&gt; deck = Riot.LoR.Deck . add_card ( deck , % Riot.LoR.Card { set : 1 , fac : 0 , num : 1 } , 3 ) %{ % Riot.LoR.Card { fac : 0 , num : 1 , set : 1 } =&gt; 3 } iex&gt; Riot.LoR.Deck . code_count ( deck ) [ { &quot;01DE001&quot; , 3 } ]","ref":"Riot.LoR.Deck.html#code_count/1","title":"Riot.LoR.Deck.code_count/1","type":"function"},{"doc":"Populate a new Riot.LoR.Deck from a list of cards and their counts. Examples iex&gt; cards = [ { &quot;01DE001&quot; , 3 } , { &quot;01DE002&quot; , 2 } ] iex&gt; Riot.LoR.Deck . from_card_counts! ( cards ) %{ % Riot.LoR.Card { fac : 0 , num : 1 , set : 1 } =&gt; 3 , % Riot.LoR.Card { fac : 0 , num : 2 , set : 1 } =&gt; 2 }","ref":"Riot.LoR.Deck.html#from_card_counts!/1","title":"Riot.LoR.Deck.from_card_counts!/1","type":"function"},{"doc":"Returns the minimum version required to support the cards in deck. If no minimum is found (e.g., the deck is empty), return the smallest version supported by the library. Examples iex&gt; cards = [ { &quot;03MT054&quot; , 1 } , { &quot;05BC198&quot; , 1 } ] iex&gt; deck = Riot.LoR.Deck . from_card_counts! ( cards ) iex&gt; Riot.LoR.Deck . min_required_version ( deck ) 4","ref":"Riot.LoR.Deck.html#min_required_version/1","title":"Riot.LoR.Deck.min_required_version/1","type":"function"},{"doc":"Create an empty Riot.LoR.Deck Examples iex&gt; Riot.LoR.Deck . new ( ) %{ }","ref":"Riot.LoR.Deck.html#new/0","title":"Riot.LoR.Deck.new/0","type":"function"},{"doc":"Remove a Riot.LoR.Card from a Riot.LoR.Deck . By default it will decrement the count by 1. You can can specify any positive integer to decrement the count by. If you pass :all or an integer greater than or equal to the current count the Riot.LoR.Card will be removed from the Riot.LoR.Deck . Examples iex&gt; cards = [ { &quot;01DE001&quot; , 3 } , { &quot;01DE002&quot; , 2 } , { &quot;01DE003&quot; , 1 } ] iex&gt; deck = Riot.LoR.Deck . from_card_counts! ( cards ) %{ % Riot.LoR.Card { fac : 0 , num : 1 , set : 1 } =&gt; 3 , % Riot.LoR.Card { fac : 0 , num : 2 , set : 1 } =&gt; 2 , % Riot.LoR.Card { fac : 0 , num : 3 , set : 1 } =&gt; 1 } iex&gt; deck = Riot.LoR.Deck . remove_card ( deck , % Riot.LoR.Card { set : 1 , fac : 0 , num : 3 } ) %{ % Riot.LoR.Card { fac : 0 , num : 1 , set : 1 } =&gt; 3 , % Riot.LoR.Card { fac : 0 , num : 2 , set : 1 } =&gt; 2 } iex&gt; deck = Riot.LoR.Deck . remove_card ( deck , % Riot.LoR.Card { set : 1 , fac : 0 , num : 2 } , 2 ) %{ % Riot.LoR.Card { fac : 0 , num : 1 , set : 1 } =&gt; 3 } iex&gt; deck = Riot.LoR.Deck . remove_card ( deck , % Riot.LoR.Card { set : 1 , fac : 0 , num : 1 } ) %{ % Riot.LoR.Card { fac : 0 , num : 1 , set : 1 } =&gt; 2 } iex&gt; Riot.LoR.Deck . remove_card ( deck , % Riot.LoR.Card { set : 1 , fac : 0 , num : 1 } , :all ) %{ }","ref":"Riot.LoR.Deck.html#remove_card/3","title":"Riot.LoR.Deck.remove_card/3","type":"function"},{"doc":"","ref":"Riot.LoR.Deck.html#t:card_counts/0","title":"Riot.LoR.Deck.card_counts/0","type":"type"},{"doc":"","ref":"Riot.LoR.Deck.html#t:t/0","title":"Riot.LoR.Deck.t/0","type":"type"},{"doc":"Functions to encode/decode a LoR deck code.","ref":"Riot.LoR.DeckCode.html","title":"Riot.LoR.DeckCode","type":"module"},{"doc":"Takes a deck code and attempts to decode into a Riot.LoR.Deck . Examples iex&gt; deck_code = &quot;CEAAAAIBAEAAC&quot; iex&gt; Riot.LoR.DeckCode . decode! ( deck_code ) %{ % Riot.LoR.Card { fac : 0 , num : 1 , set : 1 } =&gt; 1 }","ref":"Riot.LoR.DeckCode.html#decode!/1","title":"Riot.LoR.DeckCode.decode!/1","type":"function"},{"doc":"Takes a Riot.LoR.Deck and encodes it into a deck code string. Examples iex&gt; deck = Riot.LoR.Deck . new ( ) iex&gt; deck = Riot.LoR.Deck . add_card ( deck , % Riot.LoR.Card { set : 1 , fac : 0 , num : 1 } ) iex&gt; Riot.LoR.DeckCode . encode! ( deck ) &quot;CEAAAAIBAEAAC&quot;","ref":"Riot.LoR.DeckCode.html#encode!/2","title":"Riot.LoR.DeckCode.encode!/2","type":"function"},{"doc":"Static support for supported LoR Factions. Supported Factions are hard-coded into to the module. Through the functions you can fetch information by id, code, etc.","ref":"Riot.LoR.Faction.html","title":"Riot.LoR.Faction","type":"module"},{"doc":"Get a Faction id from its code. Examples iex&gt; Riot.LoR.Faction . fetch_by_code! ( &quot;DE&quot; ) { 1 , 0 , &quot;DE&quot; }","ref":"Riot.LoR.Faction.html#fetch_by_code!/1","title":"Riot.LoR.Faction.fetch_by_code!/1","type":"function"},{"doc":"Get a Faction code from its id. Examples iex&gt; Riot.LoR.Faction . fetch_by_id! ( 0 ) { 1 , 0 , &quot;DE&quot; }","ref":"Riot.LoR.Faction.html#fetch_by_id!/1","title":"Riot.LoR.Faction.fetch_by_id!/1","type":"function"},{"doc":"Get the maximum version supported. Examples iex&gt; Riot.LoR.Faction . max_version ( ) 4","ref":"Riot.LoR.Faction.html#max_version/0","title":"Riot.LoR.Faction.max_version/0","type":"function"},{"doc":"Get the minimum version supported. Examples iex&gt; Riot.LoR.Faction . min_version ( ) 1","ref":"Riot.LoR.Faction.html#min_version/0","title":"Riot.LoR.Faction.min_version/0","type":"function"},{"doc":"","ref":"Riot.LoR.Faction.html#t:t/0","title":"Riot.LoR.Faction.t/0","type":"type"},{"doc":"More on LEB128: https://en.wikipedia.org/wiki/LEB128 https://wiki.vg/Protocol#VarInt_and_VarLong https://developers.google.com/protocol-buffers/docs/encoding#varints Other LEB128 implementations: protobuf (Python): decode: https://github.com/protocolbuffers/protobuf/blob/75eff10d81233d3d08ce5ded215e526c1fcace88/python/google/protobuf/internal/decoder.py#L107-L120 encode: https://github.com/protocolbuffers/protobuf/blob/75eff10d81233d3d08ce5ded215e526c1fcace88/python/google/protobuf/internal/encoder.py#L375-L382 Golang: decode: https://pkg.go.dev/encoding/binary#Uvarint encode: https://pkg.go.dev/encoding/binary#PutUvarint","ref":"Riot.Util.Varint.LEB128.html","title":"Riot.Util.Varint.LEB128","type":"module"},{"doc":"Decodes all varint sequences Examples iex&gt; Riot.Util.Varint.LEB128 . decode_all ( &lt;&lt; 8 , 172 , 2 , 151 , 195 , 6 &gt;&gt; ) { [ 8 , 300 , 106903 ] , &quot;&quot; , 6 }","ref":"Riot.Util.Varint.LEB128.html#decode_all/2","title":"Riot.Util.Varint.LEB128.decode_all/2","type":"function"},{"doc":"Decodes the next varint sequence Examples iex&gt; { _ , rest , _ } = Riot.Util.Varint.LEB128 . decode_next ( &lt;&lt; 8 , 172 , 2 &gt;&gt; ) { 8 , &lt;&lt; 172 , 2 &gt;&gt; , 1 } iex&gt; Riot.Util.Varint.LEB128 . decode_next ( rest ) { 300 , &quot;&quot; , 2 }","ref":"Riot.Util.Varint.LEB128.html#decode_next/2","title":"Riot.Util.Varint.LEB128.decode_next/2","type":"function"},{"doc":"Encode an integer to a varint sequence Examples iex&gt; Riot.Util.Varint.LEB128 . encode ( 999 ) &lt;&lt; 231 , 7 &gt;&gt;","ref":"Riot.Util.Varint.LEB128.html#encode/2","title":"Riot.Util.Varint.LEB128.encode/2","type":"function"},{"doc":"More on VLQs https://en.wikipedia.org/wiki/Variable-length_quantity","ref":"Riot.Util.Varint.VLQ.html","title":"Riot.Util.Varint.VLQ","type":"module"},{"doc":"","ref":"Riot.Util.Varint.VLQ.html#decode_all/2","title":"Riot.Util.Varint.VLQ.decode_all/2","type":"function"},{"doc":"","ref":"Riot.Util.Varint.VLQ.html#decode_next/2","title":"Riot.Util.Varint.VLQ.decode_next/2","type":"function"},{"doc":"","ref":"Riot.Util.Varint.VLQ.html#encode/2","title":"Riot.Util.Varint.VLQ.encode/2","type":"function"},{"doc":"","ref":"Riot.Util.Varint.UnterminatedOctetSequence.html","title":"Riot.Util.Varint.UnterminatedOctetSequence","type":"exception"},{"doc":"","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"Released 2022-04-16 Initial release","ref":"changelog.html#v1-0-0","title":"Changelog - v1.0.0","type":"extras"},{"doc":"Yet another Elixir implementation of Riot 's Legends of Runterra deck code library .","ref":"readme.html","title":"Readme","type":"extras"},{"doc":"If using Hex , the package can be installed by adding :riot_lor to your list of dependencies in mix.exs : def deps do [ { :riot_lor , &quot;~&gt; 1.0.0&quot; } ] end","ref":"readme.html#installation","title":"Readme - Installation","type":"extras"},{"doc":"Documentation Docs can be found at https://hexdocs.pm/riot_lor . You can also read supplemental docs under the docs/ directory. Examples You can run the project locally with iex -S mix . You may want to configure IEx to display lists of ints as lists, rather than charlists limit inspect enum length to 4, in case your want to limit large outputs of decks Inspect options iex \\ --eval &#39;IEx.configure(inspect: [limit: 4, charlists: :as_lists])&#39; \\ -S mix iex(1)&gt; IEx . configure ( inspect : [ limit : 4 , charlists : :as_lists ] ) :ok Decoding iex(2)&gt; deck_code = &quot;CEAAECABAQJRWHBIFU2DOOYIAEBAMCIMCINCILJZAICACBANE4VCYBABAILR2HRL&quot; &quot;CEAAECABAQJRWHBIFU2DOOYIAEBAMCIMCINCILJZAICACBANE4VCYBABAILR2HRL&quot; iex(3)&gt; deck = Riot.LoR.DeckCode . decode! ( deck_code ) %{ % Riot.LoR.Card { fac : 2 , num : 6 , set : 1 } =&gt; 2 , % Riot.LoR.Card { fac : 2 , num : 9 , ... } =&gt; 2 , % Riot.LoR.Card { fac : 2 , ... } =&gt; 2 , % Riot.LoR.Card { ... } =&gt; 2 , ... } iex(4)&gt; Riot.LoR.Deck . code_count ( deck ) [ { &quot;01IO006&quot; , 2 } , { &quot;01IO009&quot; , 2 } , { &quot;01IO012&quot; , ... } , { ... } , ... ] Encoding iex(5)&gt; card_counts = [ { &quot;01DE001&quot; , 1 } , { &quot;05BC198&quot; , 1 } ] [ { &quot;01DE001&quot; , 1 } , { &quot;05BC198&quot; , 1 } ] iex(6)&gt; deck = Riot.LoR.Deck . from_card_counts! ( card_counts ) %{ % Riot.LoR.Card { fac : 0 , num : 1 , set : 1 } =&gt; 1 , % Riot.LoR.Card { fac : 10 , num : 198 , set : 5 } =&gt; 1 } iex(7)&gt; Riot.LoR.DeckCode . encode! ( deck ) &quot;CQAAAAQBAEAACAIFBLDAC&quot;","ref":"readme.html#usage","title":"Readme - Usage","type":"extras"},{"doc":"See the CONTRIBUTING.md file.","ref":"readme.html#contributing","title":"Readme - Contributing","type":"extras"},{"doc":"Licensed under the MIT License. See the LICENSE file for more details. Per Riot's Core Policies : The riot_lor project isn't endorsed by Riot Games and doesn't reflect the views or opinions of Riot Games or anyone officially involved in producing or managing Riot Games properties. Riot Games, and all associated properties are trademarks or registered trademarks of Riot Games, Inc.","ref":"readme.html#legal","title":"Readme - Legal","type":"extras"},{"doc":"","ref":"contributing.html","title":"Contributing","type":"extras"},{"doc":"A LoR deck code is a Base32 encoded binary sequence representing: The deck code format The minimum version required for the cards in deck A set of LoR card codes and their respective quantity The official library states 1 Decks are encoding via arranging VarInts (big endian) into an array and then base 32 encoding into a string. All encodings begin with 4 bits for format and 4 bits for version. You may interpret this as meaning the entirety of the byte sequence is composed of VarInts octets, including each prefix value (bundled as a single VarInt octet, which would only leave 3 bits for format). However, in the current implementation, decks are encoded by: Creating an empty list of bytes 2 Adding one byte where high 4 bits are format and low 4 bits are version 3 Adding sequences of VarInts for the cards 4 Base32 encode the byte list 5 Further, for decoding, the first 8 bits are extracted before reading VarInts 6 . Furthermore, the VarInts the official library use are actually LEB128, i.e. little-engine, not big-endian 7 . The VarInt sequence is a serialization of a hierarchical representation of card codes and their quantities. More on this in sections below.","ref":"deck_codes.html","title":"LoR deck codes","type":"extras"},{"doc":"Format The format is presumably the &quot;version&quot; for the deck code format itself, i.e. what values/ordering to expect. Currently, the only version is 1 . We can see that format is currently unused while decoding 8 . Further, is a hard-coded 9 value while encoding 10 . We can assume this value may increment if the values/order of anything for future game updates. The maximum format value is 15, since it is represented by 4 bits. This maximum is likely fixed forever since all deck codes will likely need to start by reading the format before continuing to scan. Version Riot marks versions for LoR. AFAICT , they largely signify the release of a new faction. You can see major patch releases here and faction/version mappings here Version changes are up to Riot's discretion internally. There has been some elaboration through issues 11 . Card codes Every Legends of Runeterra card has a corresponding card code. Card codes are seven character strings comprised of two characters for card set, two characters for faction identifier, and three characters for card number. -- https://github.com/RiotGames/LoRDeckCodes#cards--decks A card code is a 7 character string comprised of 3 components Card set - a 2 character, zero-padded integer Faction slug - a 2 character string Card number - a 3 character, zero-padded integer Breaking down the example &quot;01DE123&quot; ( which is not a real card ). 01 DE 055 │ │ │ │ │ │ │ │ └ ─ &quot;055&quot; is the card number . Unique to the combination of set + faction │ │ │ └ ─ &quot;DE&quot; is the faction slug . Each slug has a corresponding numerical ID │ └ ─ &quot;01&quot; is the card set Sets AFAICT, sets are releases of bundles of cards, across factions. Riot's developer portal provides assets divvied by set . Sets are distinct from version. Factions Every card belongs to one or more factions. Card codes utilize a 2 character slug. However, each slug has a corresponding integer id, which is used in the deck code encoding (described below). Card number Card numbers are unique to each combination of set and faction. They start at 1 and increment for each card in the set/faction combo. For example 01DE001 and 02DE001 are two distinct cards sharing the same card number.","ref":"deck_codes.html#concepts","title":"LoR deck codes - Concepts","type":"extras"},{"doc":"Base32 Deck code values are encoded in Base32 . AFAICT, the official library: Uses RFC 4648 alphabet 12 , i.e. /[A-Z2-7]/ Doesn't pad by default 13 14 , but can with = Format prefix The first 4 bits are the format value. Currently there is only one: 1 . Presumably the value will increment if changed in the future. Further, data following the format may depend on the format. I.e. following sections may depend on the given format. Since there's only one legal format version, we'll assume that for the following sections. Version prefix The next 4 bits are the version value. It represents the minimum game version required for the following. If/when the version is about to surpass 15, it may require a new format version. Card VarInt sequence Following the fixed prefix, is a sequence of unsigned LEB128 VarInts of arbitrary size. While the card VarInts are a list, the serialization represents a hierarchy. I imagine this an attempt represent each card in a more dense manner, i.e. shortening the total deck code as short as possible. There are 3 general levels (groupings) in the hierarchy: The card quantity (e.g., 1, 2, &amp; 3. 4+ is a special case) A combination of card set &amp; card faction The card number Any quantities &gt;=4 are not apart of the hierarchy. Instead, it's a list of quantity + card code values (e.g. [ {quant, [set, fac, num]}, ... ] ). Here's a mapping to try and help to visualize. The first key is the quantity of cards within the value. The second key is a combination of the set &amp; faction. The list value are the cards numbers. # General map { 1 : { { 1 , 2 } : [ 10 , 12 ] , { 1 , 4 } : [ 10 , 21 ] , { 2 , 4 } : [ 10 ] } , 2 : { } , 3 : { { 1 , 4 } : [ 27 ] } , 4 + : [ { 4 , [ 1 , 2 , 30 ] } , { 5 , [ 2 , 4 , 30 ] } ] } # This mapping expands (ignoring 4+) out to individual card codes like this [ &quot;01IO010&quot; , &quot;01IO012&quot; , &quot;01PZ010&quot; , &quot;01PZ21&quot; , &quot;02PZ010&quot; , &quot;01PZ027&quot; , &quot;01PZ027&quot; , &quot;01PZ027&quot; ] │ │ ├ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ┘ ├ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ┘ │ │ │ distinct combi of │ │ │ └ set + faction └ Cards repeated within quanity group │ │ │ └ The faction id converted to the slug │ └ A card code ( described above ) So how does the VarInt sequence actually represent this hierarchy? There are no special delimiters to signify the start/end of any value/group. Instead, the sequence follows a pattern. By following this pattern you can extract two general classes of values: Count of entity Literal value Literal values are grouped together. You can know how many of these values are in the group by the preceding value, representing the count. Lets take a look and break down the example deck code from the official library Deck code example from the official library page : CEAAECABAQJRWHBIFU2DOOYIAEBAMCIMCINCILJZAICACBANE4VCYBABAILR2HRL Base32 decode gives us this sequence of bytes ( represented as integers ) : &lt;&lt; 17 , 0 , 2 , 8 , 1 , 4 , 19 , 27 , 28 , 40 , 45 , 52 , 55 , 59 , 8 , 1 , 2 , 6 , 9 , 12 , 18 , 26 , 36 , 45 , 57 , 2 , 4 , 1 , 4 , 13 , 39 , 42 , 44 , 4 , 1 , 2 , 23 , 29 , 30 , 43 &gt;&gt; The first byte is 17 or 0b0001 0001 . The high 4 bits is the format and the low 4 bits is the version . Both are 1 . After popping the first byte , continuing with format 1 , we have the varint sequence representing the deck &#39;s cards. If we decode the varint sequence, we&#39; ll have this list of integers : [ 0 , 2 , 8 , 1 , 4 , 19 , 27 , 28 , 40 , 45 , 52 , 55 , 59 , 8 , 1 , 2 , 6 , 9 , 12 , 18 , 26 , 36 , 45 , 57 , 2 , 4 , 1 , 4 , 13 , 39 , 42 , 44 , 4 , 1 , 2 , 23 , 29 , 30 , 43 ] Note that the encoded / decoded sequence hold the same values . This is because all varint octets were less than 128 , i . e . the length of all Varints was 1 . An example of a LEB128 varint with a length &gt; 1 is &lt;&lt; 172 , 2 &gt;&gt; , which represents the integer value 300 So the sequence &lt;&lt; 8 , 172 , 2 , 6 &gt;&gt; would decode to [ 8 , 300 , 6 ] Lets break down each integer value [ 0 , 2 , 8 , 1 , 4 , 19 , 27 , 28 , 40 , 45 , 52 , 55 , 59 , 8 , 1 , 2 , 6 , 9 , 12 , 18 , 26 , 36 , 45 , 57 , 2 , 4 , 1 , 4 , 13 , 39 , 42 , 44 , 4 , 1 , 2 , 23 , 29 , 30 , 43 ] │ │ │ │ │ ├ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ┘ │ │ │ ├ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ┘ │ │ │ │ ├ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ┘ │ │ │ ├ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ─ ┘ │ │ │ │ │ │ │ │ │ │ │ │ │ │ │ │ │ │ │ │ │ │ │ │ └ the 8 card numbers │ │ │ └ the 8 card numbers │ │ │ │ └ the 4 card #s │ │ │ └ the 4 card nums │ │ │ │ │ │ │ │ │ │ │ │ │ │ │ │ │ │ │ └ the faction id │ │ └ the faction id │ │ │ └ the faction id │ │ └ faction id │ │ │ │ │ │ │ │ │ │ │ │ │ │ └ the set id │ └ the set id │ │ └ the set id │ └ set id │ │ │ │ │ │ │ │ │ └ there are 8 cards in the first ( 1 / 2 ) └ there are 8 cards in the first ( 1 / 2 ) │ └ there are 4 cards in └ there are 4 cards in │ │ set / faction group set / faction group │ the first ( 1 / 2 ) the second ( 2 / 2 ) │ │ │ set / faction group set / faction group │ └ num of set / faction groups with quantity 2 │ │ └ num of set / faction groups with quantity 1 └ num of set / faction groups with quantity 3 Another way to visualize is by making a tree. The first value is the index the value would appear in. You can generally read top-to-bottom, like a list, but notice how each &quot;count&quot; type value branches out to a list of &quot;literal&quot; values. As a heirarchy ( &lt; index &gt; _ &lt; kind &gt; _ &lt; value &gt; ) ├ ─ ─ 01 _format_1 ├ ─ ─ 02 _version_1 ├ ─ ─ 03 _quant3_2 │ ├ ─ ─ 04 _combi_2 │ │ ├ ─ ─ 05 _set_1 │ │ ├ ─ ─ 06 _faction_2 │ │ ├ ─ ─ 07 _card_001 │ │ └ ─ ─ 08 _card_002 │ └ ─ ─ 09 _combi_1 │ ├ ─ ─ 10 _set_2 │ ├ ─ ─ 11 _faction_9 │ └ ─ ─ 12 _card_010 ├ ─ ─ 13 _quant2_0 ├ ─ ─ 14 _quant1_1 │ └ ─ ─ 15 _combi_2 │ ├ ─ ─ 16 _set_1 │ ├ ─ ─ 17 _faction_2 │ ├ ─ ─ 18 _card_007 │ └ ─ ─ 19 _card_005 ├ ─ ─ 20 _countN_4 ├ ─ ─ 21 _set_1 ├ ─ ─ 22 _faction_2 └ ─ ─ 23 _card_020 We can see there are 2 &quot;count&quot; types Number of set/faction groups, within a given quantity group Number of cards, within a given set/faction group We create the set/faction combination by read the first two integers after the card count. After those two, ready the card numbers. Some pseduocode to represent the generic process if the quantity values were arbitrary deck_values = [ ... ] quantity_groups = [ ] while deck_values : num_sf_groups &lt;- read one integer groups = { } foreach num_sf_groups : num_cards &lt;- read one integer g_set &lt;- read one integer g_fac &lt;- read one integer foreach num_cards : card_num &lt;- read one integer groups [ { g_set , g_fac } ] . push ( card_num ) quantity_groups . push ( groups ) deck = { } for i = quantity_groups . length , j = 0 ; i &gt;= 1 ; i -- , j ++ : deck [ i ] = quantity_groups [ j ] Currently, the official implementation hard-codes quantity groups 3-1 15 , then checks for a special &quot;extended&quot; listing after group 1 16 . If the spec ever changes to arbitrary groupings, you can do something like: cards_int │ &gt; Stream . unfold ( fn [ ] -&gt; nil ints -&gt; decode_next_quantity_group ( ints ) end ) │ &gt; Enum . reverse ( ) │ &gt; Stream . with_index ( 1 ) │ &gt; Stream . map ( fn { grp , cnt } -&gt; { cnt , grp } end ) │ &gt; Map . new ( ) Essentially, expects groups quantities to be ordered in descending order from highest to 1, with contiguous group identifiers. I.e., even if a group quantity is empty, there must be a 0 to pad. So something like [1, 0, 0, 2, 1, 2] would represent quantities of 6, 5, 4, 3, 2, 1, where 5 &amp; 4 are empty. So long as they're in descending order &amp; contiguous, quantities can be of any size. And 0 padding is likely reasonable since card quantities likely won't go larger than the 10s. But it does allow for arbitrary expansion Encoding Encoding is largely just the decoding process in reverse. However, there are some notable specs to encoding to make it &quot;stable.&quot; In general, encoding &amp; decoding deck codes should be cyclical, i.e. a deck code can be decoded and re-encoded into the same code consistently, same goes for encoding and re-decoding. However, the decoding process isn't as strict as encoding process. It's possible to decode a deck code that is re-encoded into a different code. This may occur for a number of reasons. The minimum version is different. The encoding process will determine the minimum version number necessary to properly support all cards in the deck. It's possible for the to-be-decoded code to have the wrong version, since it is not checked during the decoding process The order of the card values are wrong. So long as the card values are ordered by quantity groups properly, the set/fac and number orders don't really matter for decoding. However, encoding expects a stable sort order. First by quantity, then by the number of cards in a set/fac group, then by the alpha order of the card codes . This means when two or more set/fac groups have the same number of cards, they're ordered by the card codes. This means its sorted by set number, then faction code (not id), then card number. You may not that the example code given above actually has an incorrect sort order. The code was provided by the official library before the card code sort was done, so there wasn't guaranteed stability in that order.","ref":"deck_codes.html#unpacking-deck-codes","title":"LoR deck codes - Unpacking deck codes","type":"extras"},{"doc":"Empty deck The empty deck (for format v1) is CEAAAAA . We can break components down like this: iex&gt; use Bitwise , only_operators : true iex&gt; alias Riot.Util.Varint.LEB128 iex&gt; format = 0b0001 1 iex&gt; version = 0b0001 iex&gt; prefix = ( format &lt;&lt;&lt; 4 ) ||| version 17 iex&gt; card_count_groups = List . duplicate ( 0 , 3 ) iex&gt; vals = [ prefix | card_count_groups ] iex&gt; vals [ 17 , 0 , 0 , 0 ] iex&gt; |&gt; Enum . map ( &amp; LEB128 . encode / 1 ) [ &lt;&lt; 17 &gt;&gt; , &lt;&lt; 0 &gt;&gt; , &lt;&lt; 0 &gt;&gt; , &lt;&lt; 0 &gt;&gt; ] iex&gt; |&gt; Enum . into ( &lt;&lt; &gt;&gt; ) &lt;&lt; 17 , 0 , 0 , 0 &gt;&gt; iex&gt; |&gt; Base . encode32 ( padding : false ) &quot;CEAAAAA&quot;","ref":"deck_codes.html#notable-deck-codes","title":"LoR deck codes - Notable deck codes","type":"extras"},{"doc":"&amp;#x21A9; https://github.com/RiotGames/LoRDeckCodes/blob/52d10f702e98ca048fc241622e4c7e306d826919/README.md?plain=1#L24-L26 &amp;#x21A9; https://github.com/RiotGames/LoRDeckCodes/blob/52d10f702e98ca048fc241622e4c7e306d826919/LoRDeckCodes/LoRDeckEncoder.cs#L144 &amp;#x21A9; https://github.com/RiotGames/LoRDeckCodes/blob/52d10f702e98ca048fc241622e4c7e306d826919/LoRDeckCodes/LoRDeckEncoder.cs#L149-L151 &amp;#x21A9; https://github.com/RiotGames/LoRDeckCodes/blob/52d10f702e98ca048fc241622e4c7e306d826919/LoRDeckCodes/LoRDeckEncoder.cs#L185-L191 &amp;#x21A9; https://github.com/RiotGames/LoRDeckCodes/blob/52d10f702e98ca048fc241622e4c7e306d826919/LoRDeckCodes/LoRDeckEncoder.cs#L138 &amp;#x21A9; https://github.com/RiotGames/LoRDeckCodes/blob/52d10f702e98ca048fc241622e4c7e306d826919/LoRDeckCodes/LoRDeckEncoder.cs#L78-L83 &amp;#x21A9; https://github.com/RiotGames/LoRDeckCodes/blob/52d10f702e98ca048fc241622e4c7e306d826919/LoRDeckCodes/VarintTranslator.cs#L43-L66 &amp;#x21A9; https://github.com/RiotGames/LoRDeckCodes/blob/ecfc259c7b03254e8b2776efdd4d15ea52b95f22/LoRDeckCodes/LoRDeckEncoder.cs#L81 &amp;#x21A9; https://github.com/RiotGames/LoRDeckCodes/blob/ecfc259c7b03254e8b2776efdd4d15ea52b95f22/LoRDeckCodes/LoRDeckEncoder.cs#L17 &amp;#x21A9; https://github.com/RiotGames/LoRDeckCodes/blob/ecfc259c7b03254e8b2776efdd4d15ea52b95f22/LoRDeckCodes/LoRDeckEncoder.cs#L149 &amp;#x21A9; https://github.com/RiotGames/LoRDeckCodes/issues/47 &amp;#x21A9; https://github.com/RiotGames/LoRDeckCodes/blob/52d10f702e98ca048fc241622e4c7e306d826919/LoRDeckCodes/Base32.cs#L39 &amp;#x21A9; https://github.com/RiotGames/LoRDeckCodes/blob/52d10f702e98ca048fc241622e4c7e306d826919/LoRDeckCodes/Base32.cs#L104 &amp;#x21A9; https://github.com/RiotGames/LoRDeckCodes/blob/52d10f702e98ca048fc241622e4c7e306d826919/LoRDeckCodes/Base32.cs#L146-L150 &amp;#x21A9; https://github.com/RiotGames/LoRDeckCodes/blob/ecfc259c7b03254e8b2776efdd4d15ea52b95f22/LoRDeckCodes/LoRDeckEncoder.cs#L94-L112 &amp;#x21A9; https://github.com/RiotGames/LoRDeckCodes/blob/ecfc259c7b03254e8b2776efdd4d15ea52b95f22/LoRDeckCodes/LoRDeckEncoder.cs#L115-L131","ref":"deck_codes.html#footnotes","title":"LoR deck codes - Footnotes","type":"extras"}]